# -*- encoding: utf-8 -*-
"""
sally.cli.commands module

"""
import argparse
import logging
import pprint

import falcon
from hio.core import http
from keri import help
from keri.app import directing

from sally.core import handling, httping
from sally.core.monitoring import HealthEnd

logger = help.ogler.getLogger()

parser = argparse.ArgumentParser(description='Launch SALLY sample web hook server')
parser.set_defaults(handler=lambda args: launch(args),
                    transferable=True)
parser.add_argument('-p', '--http',
                    action='store',
                    default=9923,
                    help="Port on which to listen for web hook events.  Defaults to 9923")


def launch(args, expire=0.0):
    baseFormatter = logging.Formatter('%(asctime)s [hook] %(levelname)-8s %(message)s')
    baseFormatter.default_msec_format = None
    help.ogler.baseConsoleHandler.setFormatter(baseFormatter)
    help.ogler.level = logging.getLevelName(logging.INFO)
    help.ogler.reopen(name="hook", temp=True, clear=True)

    httpPort = args.http

    app = falcon.App(
        middleware=httping.cors_middleware())
    app.add_route("/", WebhookListener())
    app.add_route("/health", HealthEnd())

    server = http.Server(port=httpPort, app=app)
    httpServerDoer = http.ServerDoer(server=server)

    logger.info(f"Sally Web Hook Sample listening on {httpPort}")
    directing.runController(doers=[httpServerDoer], expire=expire)


class WebhookListener:
    """
    Demonstration endpoint for web hook calls that prints events to stdout and stores a simple presentation cache.
    """
    def __init__(self):
        self.received = dict()

    def on_post(self, req, resp):
        """Receives web hook POST events by printing the credential results to stdout and storing presentations them in memory
        Parameters:
            req: falcon.Request HTTP request
            rep: falcon.Response HTTP response
        """
        logger.info("** HEADERS **")
        logger.info(pprint.pprint(req.headers))
        logger.info("*************")

        logger.info("**** BODY ****")
        body = req.get_media()
        logger.info(pprint.pprint(body))
        logger.info("**************")

        data = body.get("data", {})
        if not data:
            logger.error("No data in body")
            resp.media = {"error": "No data in body"}
            resp.status = falcon.HTTP_400
            return
        type = self._resolve_type(data["schema"])

        holder = data.get("recipient", "")
        presentation = dict(
            credential=data.get("credential", ""),
            type=type,
            issuer=body.get("actor", ""),
            holder=holder,
            LEI=data.get("LEI", ""),
            personLegalName=data.get("personLegalName", ""),
            officialRole=data.get("officialRole", ""),
        )
        self.received[holder] = presentation
        resp.status = falcon.HTTP_202

    def _resolve_type(self, schema_said):
        """Return human friendly name for schema type"""
        match schema_said:
            case handling.QVI_SCHEMA:
                return "QVI"
            case handling.LE_SCHEMA:
                return "LE"
            case handling.OOR_AUTH_SCHEMA:
                return "OOR Auth"
            case handling.OOR_SCHEMA:
                return "OOR"
            case _:
                raise ValueError(f"Unknown schema type with SAID: {schema_said}")

    def on_get(self, req, resp):
        """
        Tells the presenter if they have presented a credential before.
        Used in testing to determine that a presentation has succeeded.
        """
        holder = req.get_param("holder", required=True)
        if holder in self.received:
            resp.media = self.received[holder]
            resp.status = falcon.HTTP_200
        else:
            resp.media = {"error": f"No credential presented by {holder}"}
            resp.status = falcon.HTTP_404




# -*- encoding: utf-8 -*-
"""
sally.cli.commands module

"""
import argparse
import logging
import os

from keri import help
from keri.app import keeping, habbing, directing, configing, oobiing
from keri.app.cli.common import existing

import sally
from sally.core import serving

parser = argparse.ArgumentParser(description='Launch Sally vLEI credential presentation receiver service.')
parser.set_defaults(handler=lambda args: launch(args),
                    transferable=True)

parser.add_argument(
    "-d", "--direct", action="store_true", default=False,
    help="Listen for direct-mode messages on HTTP port.  Default is True")
parser.add_argument(
    '-p', '--http', action='store', default=9723,
    help="Port on which to listen for OOBI requests.  Defaults to 9723")
parser.add_argument(
    '-n', '--name', action='store', default="sally",
    help="Name of controller. Default is sally.")
parser.add_argument(
    '-b', '--base', required=False, default="",
    help='additional optional prefix to file location of KERI keystore')
parser.add_argument(
    "-c", "--config-dir", dest="configDir",
    help="directory override for configuration data")
parser.add_argument(
    "-f", '--config-file', dest="configFile", action='store', default=None,
    help="Habery (keystore) configuration filename")
parser.add_argument(
    "-i", "--incept-file", dest="inceptFile", action='store', default=None,
    help="inception configuration filename")
parser.add_argument(
    '-a', '--alias', required=True,
    help='human readable alias for the new identifier prefix')
parser.add_argument(
    '-s', '--salt', required=False,
    help='qualified base64 salt for creating key pairs')
parser.add_argument(
    '--passcode', dest="bran", default=None,
    help='21 character encryption passcode for keystore (is not saved)')
parser.add_argument(
    '-w', '--web-hook', action='store', required=True, default=None,
    help='Webhook address for outbound notifications of credential issuance or revocation')
parser.add_argument(
    '--auth', action="store", required=True,
    help='AID or alias of authority for OOBIs and QVI credential issuer')
parser.add_argument(
    "-r", "--retry-delay", default=10, type=int, action="store",
    help="retry delay (in seconds) for failed web hook attempts")
parser.add_argument(
    "-e", "--escrow-timeout", default=10, type=int, action="store",
    help="timeout (in minutes) for escrowed events that have not been delivered to the web hook.  Defaults to 10")
parser.add_argument(
    "-l", "--loglevel", action="store", required=False, default=os.getenv("SALLY_LOG_LEVEL", "INFO"),
    help="Set log level to DEBUG | INFO | WARNING | ERROR | CRITICAL. Default is CRITICAL")

help.ogler.level = logging.getLevelName(logging.INFO)
logger = help.ogler.getLogger()

def launch(args, expire=0.0):
    """Launch Sally vLEI credential presentation receiver service"""
    # Logging config
    base_formatter = logging.Formatter('%(asctime)s [sally] %(levelname)-8s %(message)s')
    base_formatter.default_msec_format = None
    help.ogler.baseConsoleHandler.setFormatter(base_formatter)
    help.ogler.level = logging.getLevelName(args.loglevel.upper())
    logger.setLevel(help.ogler.level)
    help.ogler.reopen(name="sally", temp=True, clear=True)

    hook = args.web_hook
    name = args.name
    salt = args.salt
    base = args.base
    bran = args.bran
    try:
        http_port = int(args.http)
    except ValueError:
        raise ValueError(f"Invalid port number: {args.http}. Must be an integer.")
    auth = args.auth
    direct = args.direct

    timeout = args.escrow_timeout
    retry = args.retry_delay

    alias = args.alias
    config_file = args.configFile
    config_dir = args.configDir

    # Inception configuration for when Sally bootstraps itself
    incept_file = args.inceptFile
    incept_args = {
        "name": name, "base": base, "alias": alias, "bran": bran,
        "incept_file": incept_file, "config_dir": config_dir
    }

    ks = keeping.Keeper(name=name, base=base, temp=False, reopen=True)
    aeid = ks.gbls.get('aeid')

    cf = None
    if aeid is None:
        if config_file is not None:
            cf = configing.Configer(name=config_file, base=base, headDirPath=config_dir,
                                    temp=False, reopen=True, clear=False)
        habery_cfg = dict()
        habery_cfg["salt"] = salt if salt else None # When None causes Habery to randomize salt
        hby = habbing.Habery(name=name, base=base, bran=bran, cf=cf, **habery_cfg)
    else:
        hby = existing.setupHby(name=name, base=base, bran=bran)

    # setup doers
    hbyDoer = habbing.HaberyDoer(habery=hby)
    obl = oobiing.Oobiery(hby=hby)

    doers = [hbyDoer, *obl.doers]
    doers += serving.setup(hby, alias=alias, httpPort=http_port, hook=hook, auth=auth,
                           timeout=timeout, retry=retry, direct=direct, incept_args=incept_args)

    logger.info(f"Sally Server v{sally.__version__} listening on {http_port} with DB version {hby.db.version}")
    directing.runController(doers=doers, expire=expire)
